unit Source;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IGC, CHM, PrefixTest;

type
  TSourceModule = class(TDataModule)
    hl7Engine: TPrefixTestengine;
    IguanaPlugin: TIGCsource;
    procedure IguanaPluginDisconnected(Sender: TObject);
    procedure IguanaPluginMessageRequestCompleted(Sender: TObject;
      Inputs: IIGCsourceMessageRequestCompletedEventInputs;
      Outputs: IIGCsourceMessageRequestCompletedEventOutputs);
    procedure IguanaPluginMessageRequested(Sender: TObject;
      Inputs: IIGCsourceMessageRequestedEventInputs;
      Outputs: IIGCsourceMessageRequestedEventOutputs);
    procedure hl7EngineLog(Sender: TObject; Line: String);
  private
    function GenerateRandomMessage : string;
    // Begin generation declarations (marker used by STD)
    function GenerateADTA04 : string;
    function GenerateADTA08 : string;
    // End generation declarations (marker used by STD)

    { Private declarations }
  public
    FPythonLog : String;
    procedure Run(HostName : String; Port :Integer; ChannelName : String);
    { Public declarations }
  end;

implementation

{$R *.DFM}

{ TSourceModule }

procedure TSourceModule.Run(HostName: String; Port: Integer;
  ChannelName: String);
begin
   hl7Engine.Load(ExtractFilePath(ParamStr(0)) + 'test_data.vmd');
   Write('Connecting to ' + HostName + ':' + IntToStr(Port) + ' for Channel ''' + ChannelName + '''...');
   Flush(Output);
   IguanaPlugin.Connect(HostName, Port, ChannelName);
   WriteLn('Done');
   Flush(Output);
   IguanaPlugin.WaitForDisconnect;
end;

procedure TSourceModule.IguanaPluginDisconnected(Sender: TObject);
begin
   WriteLn('Iguana disconnected from plugin.');
   Flush(Output);
end;

procedure TSourceModule.IguanaPluginMessageRequested(
  Sender : TObject;
  Inputs : IIGCsourceMessageRequestedEventInputs;
  Outputs : IIGCsourceMessageRequestedEventOutputs);
begin
   FPythonLog := '';  // clear python log
   Outputs.OutboundMessage := GenerateRandomMessage;
   // TODO - one could make the PluginMessageId's unique here - they are passed
   // back in the IguanaPluginMessageRequestCompleted method.
   Outputs.PluginMessageId := 'A';
   if (Length(FPythonLog) > 0) then
   begin
      Outputs.AddInfoLogMessage(FPythonLog);
   end;

   // If the plugin detects an error when trying to create a message, set OutboundMessage
   // to '', and add a log error message
   //
   // Outputs.OutboundMessage := '';
   // Outputs.AddErrorLogMessage('A generation error occured.');
   //
   // You can use StopChannel, StopChannelErrorFlag, and StopChannelReason to stop the Iguana
   // channel with or without an error flag, and provide a reason for the stop request.
   //
   // Outputs.StopChannelFlag := true;
   // Outputs.StopChannelErrorFlag := true;
   // Outputs.StopChannelReason := 'An error occurred.';
   //
   // You can send additional error/info/debug log messages to Iguana
   //
   // Outputs.AddWarningLogMessage('Error log message.');
   // Outputs.AddInfoMessage('Info log message.');
   // Outputs.AddDebugMessage('Debug log message.');

   WriteLn('Got message request.');
   Flush(Output);
   Sleep(2000);
   WriteLn('Sent it.');
   Flush(Output);
end;

procedure TSourceModule.IguanaPluginMessageRequestCompleted(
   Sender: TObject;
   Inputs: IIGCsourceMessageRequestCompletedEventInputs;
   Outputs: IIGCsourceMessageRequestCompletedEventOutputs);
begin
   WriteLn('Message processed: ' + Inputs.PluginMessageId);
   Sleep(1000);
end;

procedure TSourceModule.hl7EngineLog(Sender: TObject; Line: String);
begin
   FPythonLog := FPythonLog + Line + #13;
end;

// Begin GenerateRandomMessage function (anchor used by STD)
function TSourceModule.GenerateRandomMessage : string;
begin
   case Random(2) of
   1: Result := GenerateADTA04;
   2: Result := GenerateADTA08;
   else
      Result := GenerateADTA04;
   end;
end;
// End GenerateRandomMessage function (anchor used by STD)

// These generated methods are examples of how an HL7 message
// can be created using Chameleon's generated stubcode objects.
// We are populating these objects with dummy data.  In a real
// application you would need to obtain this data from somewhere
// like a database.
//
// It should be noted that each Msg object corresponds to a
// message defined in the vmd file loaded into the hl7Engine.
// These objects have tables and sub tables with properties that you
// can use to populate the message.
//
// It must be emphasized that this will be a mirror of the 'table grammar' for
// each message - i.e. the user defined definitions of each table and the fields
// in those tables as defined in the Chameleon GUI.
//
// Also keep in mind that this is using the static stubcode interface of Chameleon.
// For advanced applications it is possible to use a dynamic interface to populate
// messages.  See http://www.interfaceware.com/manual/delphi_dynamic_vs_static.html
// Begin generation functions (anchor used by STD)
function TSourceModule.GenerateADTA04 : string;
var
    Msg : TPrefixTestmsgADTA04;
begin
    Msg := TPrefixTestmsgADTA04.Create(hl7Engine);
    try
        Msg.PatientInfo.AddRow;
        Msg.PatientInfo[0].SetIDPid := 8;
        Msg.PatientInfo[0].PatientNameFamilyNameSurnamerepeatsonlyfirstinstancemapped := 'PatientNameFamilyNameSurnamerepeatsonlyfirstinstancemapped Dummy Value';
        Msg.PatientInfo[0].PatientNameGivenNamerepeatsonlyfirstinstancemapped := 'PatientNameGivenNamerepeatsonlyfirstinstancemapped Dummy Value';
        Msg.PatientInfo[0].DatetimeOfBirthTime.DateTime := Now;
        Msg.PatientInfo[0].AdministrativeSex := 'AdministrativeSex Dummy Value';
        Result := Msg.GenerateFlatWire;
    finally
        Msg.Free
    end;
end;

function TSourceModule.GenerateADTA08 : string;
var
    Msg : TPrefixTestmsgADTA08;
begin
    Msg := TPrefixTestmsgADTA08.Create(hl7Engine);
    try
        Msg.PatientInfo.AddRow;
        Msg.PatientInfo[0].SetIDPid := 8;
        Msg.PatientInfo[0].PatientNameFamilyNameSurnamerepeatsonlyfirstinstancemapped := 'PatientNameFamilyNameSurnamerepeatsonlyfirstinstancemapped Dummy Value';
        Msg.PatientInfo[0].PatientNameGivenNamerepeatsonlyfirstinstancemapped := 'PatientNameGivenNamerepeatsonlyfirstinstancemapped Dummy Value';
        Msg.PatientInfo[0].DatetimeOfBirthTime.DateTime := Now;
        Msg.PatientInfo[0].AdministrativeSex := 'AdministrativeSex Dummy Value';
        Result := Msg.GenerateFlatWire;
    finally
        Msg.Free
    end;
end;
// End generation functions (anchor used by STD)

end.
